/********************************************************
 * MAT259 PROJ.1b   2D Matrix                           *
 *                                                      *
 * Junxiang Yao                                         *     
 *                                                      *                          
 ********************************************************/
//layout 
int horizontalMargin = 150;
int verticalMargin = 140;

//data matrix related
Table table;
int rows, cols;
float[][] dataMatrix;
float max, min;
float cellWidth, cellHeight;
float r;

//color scheme
PImage colorBar;

//icons and names of data fields
PImage[] images;
String[] f = {"Basketball", "Volleyball", "Football", 
  "Tennis", "Table Tennis", "Golf", "Baseball", "Weight Lifting\nTrack&Field\nGymnastics", 
  "Cycling", "Motor", "Combat", "Ice&Snow", "Boating", "Swimming\n& Diving", };

// initialize the year
int startYear = 2006;

void setup() {
  size(1280, 720);
  surface.setResizable(true);
  //initialize the data
  table = loadTable("AllSports.csv", "header");
  rows = table.getRowCount(); 
  cols = table.getColumnCount();
  dataMatrix = new float[cols-2][rows];
  max = 0;
  for (int i=0; i<cols-2; i++) {
    for (int j=0; j<rows; j++) {
      dataMatrix[i][j] = table.getFloat(j, i+2);
      //find the max value
      if (dataMatrix[i][j]>max) {
        max = dataMatrix[i][j];
      }
    }
  }
  //find the min value
  min = max;
  for (int i=0; i<cols-2; i++) {
    for (int j=0; j<rows; j++) {
      if (dataMatrix[i][j]<min) {
        min = dataMatrix[i][j];
      }
    }
  }
print(max);

  // using colors from colorbar
  colorBar = loadImage("colorBar.jpg");

  //image array for the icons
  images = new PImage[cols-2];
  for (int i = 0; i < images.length; i++ ) {
    images[i] = loadImage( i + ".png" );
  }
}


void draw() {
  smooth();
  //refresh the canvas every frame
  background(40);

  //modifying the cell size according to window size
  cellWidth = (width - horizontalMargin*2)/rows;
  cellHeight = (height - verticalMargin*2)/cols;
  r = cellWidth/2;

  //set the position of the years
  for (int i=0; i<9; i++) {
    textAlign(CENTER, CENTER);
    fill(120);
    textSize(22);
    text(startYear+i, horizontalMargin + 12*cellWidth*i + cellWidth*6-3, 
      height - verticalMargin + 10 );
      //brighten the background to emphasize the years when mouse is hovering
    if (mouseX>(horizontalMargin + 12*cellWidth*i + cellWidth/2-cellWidth+2)&&
      mouseX< (horizontalMargin + 12*cellWidth*i + cellWidth/2-cellWidth+2+12*cellWidth) 
      && mouseY>(height - verticalMargin -20) && mouseY < height-verticalMargin+48) {
      fill(255, 10);
      noStroke();
      rect(horizontalMargin + 12*cellWidth*i + cellWidth/2-cellWidth+2, 180, cellWidth*12, height - verticalMargin*2 + 2);
    }
  }

  //draw 2D Matrix
  for (int i=0; i<cols-2; i++) {
    for (int j=0; j<rows; j++) {
      noStroke();
      color c = colorBar.get((int)map(dataMatrix[i][j], min, max, (colorBar.width-2)*5/6, 2), colorBar.height/2);
      fill(c);
      //except for the color, size is another argument while displaying data
      //the brighter the color is, the larger the ellipse will be
      float size = map(brightness(c), 0, 255, -3, 5);
      ellipse(horizontalMargin + cellWidth*j + r/2, verticalMargin + cellHeight*i + cellHeight/2 +50, r*size, r*size);
    }
  }
  
  //draw the icons from the image array
  for (int i=0; i<cols-2; i++) {
    imageMode(CENTER);
    image(images[i], horizontalMargin-cellWidth*2, verticalMargin + cellHeight*i + cellHeight/2+50, 20, 20);
  }

  // set the text version of the name of the rows by the left side of the icons, and
  // display when the position of the mouse is appropriate.
  if (horizontalMargin-cellWidth*2-10<mouseX&&horizontalMargin-cellWidth*2+10>mouseX) {
    for (int i = 0; i < cols-2; i++) {
      if (mouseY>verticalMargin + cellHeight*i + cellHeight/2+40&&
        mouseY<verticalMargin + cellHeight*(i+1) + cellHeight/2+40) {
        textAlign(RIGHT, CENTER);
        fill(100);
        textSize(17);
        text(f[i], horizontalMargin-cellWidth*2-15, verticalMargin + cellHeight*i + cellHeight/2 + 47);
      }
    }
  }
  
  
  //header
  textAlign(LEFT, CENTER);
  fill(200);
  textSize(36);
  text("How Reading Preferences of Books about",
  horizontalMargin-cellWidth*2+10,50);
  text("Sports in the Library Varies with Time",
  horizontalMargin-cellWidth*2+10,90);
  fill(120);
  textSize(20);
  text("Checkout times of the sports related books in the Seattle Public Library",
  horizontalMargin-cellWidth*2+10,130);
  noStroke();
  fill(120);
  rect(horizontalMargin-cellWidth*2-cellWidth*2+5,180,width - horizontalMargin*2+20, 3);
  
  //author
  textAlign(RIGHT, CENTER);
  textSize(12);
  text("Junxiang Yao", 
  width - horizontalMargin-12 ,151);
  text("MAT259 PROJ 1b", 
  width - horizontalMargin-12 ,167);
  
  
  //Colorbar
  textAlign(LEFT);
  textSize(12);
  text("Numbers of CheckOuts", 
  horizontalMargin-cellWidth*2-cellWidth*2+10,height - verticalMargin + 80);
  

  imageMode(CENTER);
  image(colorBar,horizontalMargin-cellWidth*2-cellWidth*2 + 230,
  height - verticalMargin + 76,150,12);
  
  textAlign(LEFT);
  textSize(12);
  text("443", 
  horizontalMargin-cellWidth*2-cellWidth*2+155,height - verticalMargin + 100);
  text("0", 
  horizontalMargin-cellWidth*2-cellWidth*2+305,height - verticalMargin + 100);
}